@@ -330,7 +330,7 @@ static PHP_METHOD(amqp_channel_class, __construct)
             "Could not create channel. No connection resource.",
             0
         );
-        return;
+        RETURN_THROWS();
     }
 
     if (!connection->connection_resource->is_connected) {
@@ -515,7 +515,7 @@ static PHP_METHOD(amqp_channel_class, setPrefetchCount)
     zend_long prefetch_count;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l", &prefetch_count) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     if (!php_amqp_is_valid_prefetch_count(prefetch_count)) {
@@ -603,7 +603,7 @@ static PHP_METHOD(amqp_channel_class, setPrefetchSize)
     zend_long prefetch_size;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l", &prefetch_size) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     if (!php_amqp_is_valid_prefetch_size(prefetch_size)) {
@@ -688,7 +688,7 @@ static PHP_METHOD(amqp_channel_class, setGlobalPrefetchCount)
     zend_long global_prefetch_count;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l", &global_prefetch_count) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     if (!php_amqp_is_valid_prefetch_count(global_prefetch_count)) {
@@ -755,7 +755,7 @@ static PHP_METHOD(amqp_channel_class, setGlobalPrefetchSize)
     zend_long global_prefetch_size;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l", &global_prefetch_size) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     if (!php_amqp_is_valid_prefetch_size(global_prefetch_size)) {
@@ -826,7 +826,7 @@ static PHP_METHOD(amqp_channel_class, qos)
     bool global = 0;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "ll|b", &prefetch_size, &prefetch_count, &global) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(getThis());
@@ -913,9 +913,7 @@ static PHP_METHOD(amqp_channel_class, startTransaction)
 
     amqp_rpc_reply_t res;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "") == FAILURE) {
-        return;
-    }
+    PHP_AMQP_NOPARAMS()
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(getThis());
     PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not start the transaction.");
@@ -943,9 +941,7 @@ static PHP_METHOD(amqp_channel_class, commitTransaction)
 
     amqp_rpc_reply_t res;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "") == FAILURE) {
-        return;
-    }
+    PHP_AMQP_NOPARAMS()
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(getThis());
     PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not start the transaction.");
@@ -972,10 +968,7 @@ static PHP_METHOD(amqp_channel_class, rollbackTransaction)
 
     amqp_rpc_reply_t res;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "") == FAILURE) {
-        return;
-    }
-
+    PHP_AMQP_NOPARAMS()
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(getThis());
     PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not rollback the transaction.");
@@ -1015,7 +1008,7 @@ static PHP_METHOD(amqp_channel_class, basicRecover)
     bool requeue = 1;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "|b", &requeue) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(getThis());
@@ -1046,9 +1039,7 @@ PHP_METHOD(amqp_channel_class, confirmSelect)
     amqp_channel_resource *channel_resource;
     amqp_rpc_reply_t res;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "") == FAILURE) {
-        return;
-    }
+    PHP_AMQP_NOPARAMS()
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(getThis());
     PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not enable confirms mode.");
@@ -1060,6 +1051,7 @@ PHP_METHOD(amqp_channel_class, confirmSelect)
     if (PHP_AMQP_MAYBE_ERROR(res, channel_resource)) {
         php_amqp_zend_throw_exception_short(res, amqp_channel_exception_class_entry);
         php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
+
         return;
     }
 
@@ -1075,7 +1067,7 @@ PHP_METHOD(amqp_channel_class, setReturnCallback)
     zend_fcall_info_cache fcc = empty_fcall_info_cache;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "f!", &fci, &fcc) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     amqp_channel_object *channel = PHP_AMQP_GET_CHANNEL(getThis());
@@ -1106,7 +1098,7 @@ PHP_METHOD(amqp_channel_class, waitForBasicReturn)
     struct timeval *tv_ptr = &tv;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "|d", &timeout) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     if (timeout < 0) {
@@ -1170,13 +1162,7 @@ PHP_METHOD(amqp_channel_class, waitForBasicReturn)
             return;
         }
 
-        status = php_amqp_handle_basic_return(
-            &PHP_AMQP_G(error_message),
-            channel_resource->connection_resource,
-            channel_resource->channel_id,
-            channel,
-            &method
-        );
+        status = php_amqp_handle_basic_return(&PHP_AMQP_G(error_message), channel, &method);
 
         if (PHP_AMQP_RESOURCE_RESPONSE_BREAK == status) {
             php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
@@ -1215,7 +1201,7 @@ PHP_METHOD(amqp_channel_class, setConfirmCallback)
     zend_fcall_info_cache nack_fcc = empty_fcall_info_cache;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "f!|f!", &ack_fci, &ack_fcc, &nack_fci, &nack_fcc) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     amqp_channel_object *channel = PHP_AMQP_GET_CHANNEL(getThis());
@@ -1255,7 +1241,7 @@ PHP_METHOD(amqp_channel_class, waitForConfirm)
     struct timeval *tv_ptr = &tv;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "|d", &timeout) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     if (timeout < 0) {
@@ -1325,31 +1311,13 @@ PHP_METHOD(amqp_channel_class, waitForConfirm)
 
         switch (method.id) {
             case AMQP_BASIC_ACK_METHOD:
-                status = php_amqp_handle_basic_ack(
-                    &PHP_AMQP_G(error_message),
-                    channel_resource->connection_resource,
-                    channel_resource->channel_id,
-                    channel,
-                    &method
-                );
+                status = php_amqp_handle_basic_ack(&PHP_AMQP_G(error_message), channel, &method);
                 break;
             case AMQP_BASIC_NACK_METHOD:
-                status = php_amqp_handle_basic_nack(
-                    &PHP_AMQP_G(error_message),
-                    channel_resource->connection_resource,
-                    channel_resource->channel_id,
-                    channel,
-                    &method
-                );
+                status = php_amqp_handle_basic_nack(&PHP_AMQP_G(error_message), channel, &method);
                 break;
             case AMQP_BASIC_RETURN_METHOD:
-                status = php_amqp_handle_basic_return(
-                    &PHP_AMQP_G(error_message),
-                    channel_resource->connection_resource,
-                    channel_resource->channel_id,
-                    channel,
-                    &method
-                );
+                status = php_amqp_handle_basic_return(&PHP_AMQP_G(error_message), channel, &method);
                 break;
             default:
                 status = AMQP_STATUS_WRONG_METHOD;
