@@ -104,7 +104,7 @@ int php_amqp_connection_resource_error(
             return PHP_AMQP_RESOURCE_RESPONSE_ERROR;
 
         case AMQP_RESPONSE_LIBRARY_EXCEPTION:
-            spprintf(message, 0, "Library error: %s", amqp_error_string2(reply.library_error));
+            spprintf(message, 0, "%s", amqp_error_string2(reply.library_error));
             return PHP_AMQP_RESOURCE_RESPONSE_ERROR;
 
         case AMQP_RESPONSE_SERVER_EXCEPTION:
@@ -234,13 +234,7 @@ static void php_amqp_close_channel_from_server(
 }
 
 
-int php_amqp_connection_resource_error_advanced(
-    amqp_rpc_reply_t reply,
-    char **message,
-    amqp_connection_resource *resource,
-    amqp_channel_t channel_id,
-    amqp_channel_object *channel
-)
+int php_amqp_connection_resource_error_advanced(amqp_rpc_reply_t reply, char **message, amqp_channel_object *channel)
 {
     assert(resource != NULL);
 
@@ -249,28 +243,35 @@ int php_amqp_connection_resource_error_advanced(
     assert(AMQP_RESPONSE_LIBRARY_EXCEPTION == reply.reply_type);
     assert(AMQP_STATUS_UNEXPECTED_STATE == reply.library_error);
 
-    if (channel_id < 0 || AMQP_STATUS_OK != amqp_simple_wait_frame(resource->connection_state, &frame)) {
+    if (channel->channel_resource->channel_id < 0 ||
+        AMQP_STATUS_OK !=
+            amqp_simple_wait_frame(channel->channel_resource->connection_resource->connection_state, &frame)) {
         if (*message != NULL) {
             efree(*message);
         }
 
-        spprintf(message, 0, "Library error: %s", amqp_error_string2(reply.library_error));
+        spprintf(message, 0, "%s", amqp_error_string2(reply.library_error));
         return PHP_AMQP_RESOURCE_RESPONSE_ERROR;
     }
 
-    if (channel_id != frame.channel) {
-        spprintf(message, 0, "Library error: channel mismatch");
+    if (channel->channel_resource->channel_id != frame.channel) {
+        spprintf(message, 0, "Channel mismatch");
         return PHP_AMQP_RESOURCE_RESPONSE_ERROR;
     }
 
     if (AMQP_FRAME_METHOD == frame.frame_type) {
         switch (frame.payload.method.id) {
             case AMQP_CONNECTION_CLOSE_METHOD: {
-                php_amqp_close_connection_from_server(reply, message, resource);
+                php_amqp_close_connection_from_server(reply, message, channel->channel_resource->connection_resource);
                 return PHP_AMQP_RESOURCE_RESPONSE_ERROR_CONNECTION_CLOSED;
             }
             case AMQP_CHANNEL_CLOSE_METHOD: {
-                php_amqp_close_channel_from_server(reply, message, resource, channel_id);
+                php_amqp_close_channel_from_server(
+                    reply,
+                    message,
+                    channel->channel_resource->connection_resource,
+                    channel->channel_resource->channel_id
+                );
                 return PHP_AMQP_RESOURCE_RESPONSE_ERROR_CHANNEL_CLOSED;
             }
 
@@ -279,30 +280,25 @@ int php_amqp_connection_resource_error_advanced(
 				 * here is a message being confirmed
 				 */
 
-                return php_amqp_handle_basic_ack(message, resource, channel_id, channel, &frame.payload.method);
+                return php_amqp_handle_basic_ack(message, channel, &frame.payload.method);
             case AMQP_BASIC_NACK_METHOD:
                 /* if we've turned publisher confirms on, and we've published a message
 				 * here is a message being confirmed
 				 */
 
-                return php_amqp_handle_basic_nack(message, resource, channel_id, channel, &frame.payload.method);
+                return php_amqp_handle_basic_nack(message, channel, &frame.payload.method);
             case AMQP_BASIC_RETURN_METHOD:
                 /* if a published message couldn't be routed and the mandatory flag was set
 				 * this is what would be returned. The message then needs to be read.
 				 */
 
-                return php_amqp_handle_basic_return(message, resource, channel_id, channel, &frame.payload.method);
+                return php_amqp_handle_basic_return(message, channel, &frame.payload.method);
             default:
                 if (*message != NULL) {
                     efree(*message);
                 }
 
-                spprintf(
-                    message,
-                    0,
-                    "Library error: An unexpected method was received 0x%08X\n",
-                    frame.payload.method.id
-                );
+                spprintf(message, 0, "An unexpected method was received 0x%08X\n", frame.payload.method.id);
                 return PHP_AMQP_RESOURCE_RESPONSE_ERROR;
         }
     }
@@ -311,7 +307,7 @@ int php_amqp_connection_resource_error_advanced(
         efree(*message);
     }
 
-    spprintf(message, 0, "Library error: %s", amqp_error_string2(reply.library_error));
+    spprintf(message, 0, "%s", amqp_error_string2(reply.library_error));
     return PHP_AMQP_RESOURCE_RESPONSE_ERROR;
 }
 
@@ -367,7 +363,7 @@ int php_amqp_set_resource_rpc_timeout(amqp_connection_resource *resource, double
     rpc_timeout.tv_usec = (int) ((timeout - floor(timeout)) * 1.e+6);
 
     if (AMQP_STATUS_OK != amqp_set_rpc_timeout(resource->connection_state, &rpc_timeout)) {
-        zend_throw_exception(amqp_connection_exception_class_entry, "Library error: cannot set rpc_timeout", 0);
+        zend_throw_exception(amqp_connection_exception_class_entry, "Cannot set rpc_timeout", 0);
         return 0;
     }
 #endif
@@ -644,7 +640,7 @@ amqp_connection_resource *connection_resource_constructor(amqp_connection_params
 
         php_amqp_connection_resource_error(res, &message, resource, 0);
 
-        spprintf(&long_message, 0, "%s - Potential login failure.", message);
+        spprintf(&long_message, 0, "%s", message);
         zend_throw_exception(amqp_connection_exception_class_entry, long_message, PHP_AMQP_G(error_code));
 
         efree(message);
