@@ -62,6 +62,9 @@
 #include "amqp_envelope.h"
 #include "amqp_queue.h"
 #include "amqp_type.h"
+#include "amqp_value.h"
+#include "amqp_decimal.h"
+#include "amqp_timestamp.h"
 
 zend_class_entry *amqp_queue_class_entry;
 #define this_ce amqp_queue_class_entry
@@ -80,7 +83,7 @@ static PHP_METHOD(amqp_queue_class, __construct)
     amqp_channel_resource *channel_resource;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &channelObj, amqp_channel_class_entry) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     ZVAL_UNDEF(&arguments);
@@ -127,7 +130,7 @@ static PHP_METHOD(amqp_queue_class, setName)
     size_t name_len = 0;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &name, &name_len) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     if (name_len < 1 || name_len > 255) {
@@ -185,7 +188,7 @@ static PHP_METHOD(amqp_queue_class, setFlags)
     bool flags_is_null = 1;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l!", &flags, &flags_is_null) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     flags = flags & PHP_AMQP_QUEUE_FLAGS;
@@ -208,7 +211,7 @@ static PHP_METHOD(amqp_queue_class, getArgument)
     size_t key_len;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &key, &key_len) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     if ((tmp = zend_hash_str_find(PHP_AMQP_READ_THIS_PROP_ARR("arguments"), key, key_len)) == NULL) {
@@ -229,7 +232,7 @@ static PHP_METHOD(amqp_queue_class, hasArgument)
     size_t key_len;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &key, &key_len) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     RETURN_BOOL(zend_hash_str_find(PHP_AMQP_READ_THIS_PROP_ARR("arguments"), key, key_len) != NULL);
@@ -254,7 +257,7 @@ static PHP_METHOD(amqp_queue_class, setArguments)
     zval *zvalArguments;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "a/", &zvalArguments) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     zend_update_property(this_ce, PHP_AMQP_COMPAT_OBJ_P(getThis()), ZEND_STRL("arguments"), zvalArguments);
@@ -273,23 +276,33 @@ static PHP_METHOD(amqp_queue_class, setArgument)
     zval *value = NULL;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "sz", &key, &key_len, &value) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     switch (Z_TYPE_P(value)) {
+        case IS_OBJECT:
+            if (!instanceof_function(Z_OBJCE_P(value), amqp_timestamp_class_entry) &&
+                !instanceof_function(Z_OBJCE_P(value), amqp_decimal_class_entry) &&
+                !instanceof_function(Z_OBJCE_P(value), amqp_value_class_entry)) {
+                goto err;
+            }
+            // Intentional fall-through
         case IS_NULL:
         case IS_TRUE:
         case IS_FALSE:
         case IS_LONG:
         case IS_DOUBLE:
         case IS_STRING:
+        case IS_ARRAY:
             zend_hash_str_add(PHP_AMQP_READ_THIS_PROP_ARR("arguments"), key, key_len, value);
             Z_TRY_ADDREF_P(value);
             break;
         default:
+        err:
             zend_throw_exception(
                 amqp_queue_exception_class_entry,
-                "The value parameter must be of type bool, int, double, string, or null.",
+                "The value parameter must be of type bool, int, double, string, null, array, AMQPTimestamp, "
+                "AMQPDecimal, or an implementation of AMQPValue.",
                 0
             );
             return;
@@ -308,7 +321,7 @@ static PHP_METHOD(amqp_queue_class, removeArgument)
     size_t key_len = 0;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &key, &key_len) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     zend_hash_str_del_ind(PHP_AMQP_READ_THIS_PROP_ARR("arguments"), key, key_len);
@@ -404,7 +417,7 @@ static PHP_METHOD(amqp_queue_class, bind)
             &keyname_len,
             &zvalArguments
         ) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(PHP_AMQP_READ_THIS_PROP("channel"));
@@ -456,7 +469,7 @@ static PHP_METHOD(amqp_queue_class, get)
     bool flags_is_null = 1;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "|l!", &flags, &flags_is_null) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(PHP_AMQP_READ_THIS_PROP("channel"));
@@ -557,7 +570,7 @@ static PHP_METHOD(amqp_queue_class, consume)
             &consumer_tag,
             &consumer_tag_len
         ) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     zval *channel_zv = PHP_AMQP_READ_THIS_PROP("channel");
@@ -579,7 +592,7 @@ static PHP_METHOD(amqp_queue_class, consume)
     PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not get channel.");
 
     if (!(AMQP_JUST_CONSUME & flags)) {
-        /* Setup the consume */
+        /* Set up the consume loop */
         arguments = php_amqp_type_convert_zval_to_amqp_table(PHP_AMQP_READ_THIS_PROP("arguments"));
 
         amqp_basic_consume_ok_t *r = amqp_basic_consume(
@@ -665,32 +678,29 @@ static PHP_METHOD(amqp_queue_class, consume)
         amqp_rpc_reply_t res =
             amqp_consume_message(channel_resource->connection_resource->connection_state, &envelope, tv_ptr, 0);
 
-        if (AMQP_RESPONSE_LIBRARY_EXCEPTION == res.reply_type && AMQP_STATUS_TIMEOUT == res.library_error) {
-            zend_throw_exception(amqp_queue_exception_class_entry, "Consumer timeout exceed", 0);
+        if (AMQP_RESPONSE_NORMAL != res.reply_type) {
 
-            amqp_destroy_envelope(&envelope);
-            php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-            return;
-        }
+            if (AMQP_RESPONSE_LIBRARY_EXCEPTION == res.reply_type) {
 
-        if (PHP_AMQP_MAYBE_ERROR_RECOVERABLE(res, channel_resource)) {
+                // Special case consumer timeout: do not close connection but end consumption
+                if (AMQP_STATUS_TIMEOUT == res.library_error) {
+                    zend_throw_exception(amqp_queue_exception_class_entry, "Consumer timeout exceed", 0);
+                    php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
+                    return;
+                }
 
-            if (PHP_AMQP_IS_ERROR_RECOVERABLE(res, channel_resource, channel)) {
-                /* In case no message was received, continue the loop */
-                amqp_destroy_envelope(&envelope);
-
-                continue;
-            } else {
-                /* Mark connection resource as closed to prevent sending any further requests */
-                channel_resource->connection_resource->is_connected = '\0';
-
-                /* Close connection with all its channels */
-                php_amqp_disconnect_force(channel_resource->connection_resource);
+                // Handle a potentially recoverable error
+                if (AMQP_STATUS_UNEXPECTED_STATE == res.library_error &&
+                    PHP_AMQP_RESOURCE_RESPONSE_OK <=
+                        php_amqp_connection_resource_error_advanced(res, &PHP_AMQP_G(error_message), channel)) {
+                    continue;
+                }
             }
 
+            /* Mark connection resource as closed to prevent sending any further requests */
+            channel_resource->connection_resource->is_connected = '\0';
+            php_amqp_disconnect_force(channel_resource->connection_resource);
             php_amqp_zend_throw_exception_short(res, amqp_queue_exception_class_entry);
-
-            amqp_destroy_envelope(&envelope);
             php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
 
             return;
@@ -825,7 +835,7 @@ static PHP_METHOD(amqp_queue_class, ack)
     bool flags_is_null = 1;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|l!", &deliveryTag, &flags, &flags_is_null) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(PHP_AMQP_READ_THIS_PROP("channel"));
@@ -874,7 +884,7 @@ static PHP_METHOD(amqp_queue_class, nack)
     bool flags_is_null = 1;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|l!", &deliveryTag, &flags, &flags_is_null) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(PHP_AMQP_READ_THIS_PROP("channel"));
@@ -924,7 +934,7 @@ static PHP_METHOD(amqp_queue_class, reject)
     bool flags_is_null = 1;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|l!", &deliveryTag, &flags, &flags_is_null) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(PHP_AMQP_READ_THIS_PROP("channel"));
@@ -1014,7 +1024,7 @@ static PHP_METHOD(amqp_queue_class, cancel)
     size_t consumer_tag_len = 0;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "|s", &consumer_tag, &consumer_tag_len) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     zval *channel_zv = PHP_AMQP_READ_THIS_PROP("channel");
@@ -1071,6 +1081,50 @@ static PHP_METHOD(amqp_queue_class, cancel)
 }
 /* }}} */
 
+/* {{{ proto int AMQPQueue::recover([boolean requeue]);
+recover messages already delivered to the consumer
+*/
+static PHP_METHOD(amqp_queue_class, recover)
+{
+    zval rv;
+
+    amqp_channel_resource *channel_resource;
+
+    bool requeue = 1;
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS(), "|b", &requeue) == FAILURE) {
+        RETURN_THROWS();
+    }
+
+    zval *channel_zv = PHP_AMQP_READ_THIS_PROP("channel");
+
+    channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(channel_zv);
+    PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not recover messages.");
+
+    amqp_basic_recover_ok_t *r = amqp_basic_recover(
+        channel_resource->connection_resource->connection_state,
+        channel_resource->channel_id,
+        requeue
+    );
+
+    if (!r) {
+        amqp_rpc_reply_t res = amqp_get_rpc_reply(channel_resource->connection_resource->connection_state);
+
+        php_amqp_error(res, &PHP_AMQP_G(error_message), channel_resource->connection_resource, channel_resource);
+
+        php_amqp_zend_throw_exception(
+            res,
+            amqp_queue_exception_class_entry,
+            PHP_AMQP_G(error_message),
+            PHP_AMQP_G(error_code)
+        );
+        php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
+        return;
+    }
+
+    php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
+}
+/* }}} */
 
 /* {{{ proto int AMQPQueue::unbind(string exchangeName, [string routingKey, array arguments]);
 unbind queue from exchange
@@ -1098,7 +1152,7 @@ static PHP_METHOD(amqp_queue_class, unbind)
             &keyname_len,
             &zvalArguments
         ) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(PHP_AMQP_READ_THIS_PROP("channel"));
@@ -1149,7 +1203,7 @@ static PHP_METHOD(amqp_queue_class, delete)
     zend_long message_count;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "|l!", &flags, &flags_is_null) == FAILURE) {
-        return;
+        RETURN_THROWS();
     }
 
     channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(PHP_AMQP_READ_THIS_PROP("channel"));
@@ -1293,6 +1347,10 @@ ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(arginfo_amqp_queue_class_reject, ZEND_SE
     ZEND_ARG_TYPE_INFO_WITH_DEFAULT_VALUE(0, flags, IS_LONG, 1, "null")
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(arginfo_amqp_queue_class_recover, ZEND_SEND_BY_VAL, 0, IS_VOID, 0)
+    ZEND_ARG_TYPE_INFO_WITH_DEFAULT_VALUE(0, requeue, _IS_BOOL, 0, "true")
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(arginfo_amqp_queue_class_purge, ZEND_SEND_BY_VAL, 0, IS_LONG, 0)
 ZEND_END_ARG_INFO()
 
@@ -1344,6 +1402,7 @@ zend_function_entry amqp_queue_class_functions[] = {
     PHP_ME(amqp_queue_class, ack,				arginfo_amqp_queue_class_ack,				ZEND_ACC_PUBLIC)
     PHP_ME(amqp_queue_class, nack,				arginfo_amqp_queue_class_nack,				ZEND_ACC_PUBLIC)
     PHP_ME(amqp_queue_class, reject,			arginfo_amqp_queue_class_reject,			ZEND_ACC_PUBLIC)
+    PHP_ME(amqp_queue_class, recover,				arginfo_amqp_queue_class_recover,				ZEND_ACC_PUBLIC)
     PHP_ME(amqp_queue_class, purge,				arginfo_amqp_queue_class_purge,				ZEND_ACC_PUBLIC)
 
     PHP_ME(amqp_queue_class, cancel,			arginfo_amqp_queue_class_cancel,			ZEND_ACC_PUBLIC)
