@@ -34,7 +34,6 @@ extern zend_class_entry *amqp_exception_class_entry, *amqp_connection_exception_
     *amqp_channel_exception_class_entry, *amqp_exchange_exception_class_entry, *amqp_queue_exception_class_entry,
     *amqp_value_exception_class_entry;
 
-
 typedef struct _amqp_connection_resource amqp_connection_resource;
 typedef struct _amqp_connection_object amqp_connection_object;
 typedef struct _amqp_channel_object amqp_channel_object;
@@ -74,6 +73,12 @@ extern zend_module_entry amqp_module_entry;
         ZEND_ARG_TYPE_INFO(pass_by_ref, name, type_hint, allow_null)
     #define PHP_AMQP_DECLARE_PROPERTY_TYPE(type, nullable) ZEND_TYPE_ENCODE(type, nullable)
     #define PHP_AMQP_DECLARE_PROPERTY_OBJ_TYPE(class_name, nullable) ZEND_TYPE_ENCODE_CLASS(class_name, nullable)
+    #define RETURN_THROWS()                                                                                            \
+        do {                                                                                                           \
+            ZEND_ASSERT(EG(exception));                                                                                \
+            (void) return_value;                                                                                       \
+            return;                                                                                                    \
+        } while (0)
 #endif
 #if PHP_VERSION_ID < 80200
     #define zend_ini_parse_quantity_warn(v, name) (zend_atol(ZSTR_VAL(v), ZSTR_LEN(v)))
@@ -367,26 +372,6 @@ static inline amqp_channel_object *php_amqp_channel_object_fetch(zend_object *ob
      PHP_AMQP_RESOURCE_RESPONSE_OK !=                                                                                  \
          php_amqp_error(res, &PHP_AMQP_G(error_message), (channel_resource)->connection_resource, (channel_resource)))
 
-#define PHP_AMQP_MAYBE_ERROR_RECOVERABLE(res, channel_resource)                                                        \
-    ((AMQP_RESPONSE_NORMAL != (res).reply_type) &&                                                                     \
-     PHP_AMQP_RESOURCE_RESPONSE_OK != php_amqp_error_advanced(                                                         \
-                                          res,                                                                         \
-                                          &PHP_AMQP_G(error_message),                                                  \
-                                          (channel_resource)->connection_resource,                                     \
-                                          (channel_resource),                                                          \
-                                          0                                                                            \
-                                      ))
-
-#define PHP_AMQP_IS_ERROR_RECOVERABLE(res, channel_resource, channel_object)                                           \
-    (AMQP_RESPONSE_LIBRARY_EXCEPTION == (res).reply_type && AMQP_STATUS_UNEXPECTED_STATE == (res).library_error &&     \
-     (0 <= php_amqp_connection_resource_error_advanced(                                                                \
-               res,                                                                                                    \
-               &PHP_AMQP_G(error_message),                                                                             \
-               (channel_resource)->connection_resource,                                                                \
-               (amqp_channel_t) (channel_resource ? (channel_resource)->channel_id : 0),                               \
-               (channel_object)                                                                                        \
-           )))
-
 
 #if ZEND_MODULE_API_NO >= 20100000
     #define AMQP_OBJECT_PROPERTIES_INIT(obj, ce) object_properties_init(&(obj), ce);
